# Author: Ilya Grigorik
# Web: www.igvita.com
# Date: June 08, 2008
#
# Renders quotes from any tumblr account into 680x480 compliant JPG's.
# - quick and dirty, but it works. ;)
#

require 'rubygems'
require 'hpricot'
require 'open-uri'
require 'cgi'
require 'rvg/rvg'
include Magick

# courtesy of http://blog.choonkeat.com/weblog/2007/02/rmagick-renderi.html
module RMagickTextUtil
    def render_cropped_text(caption_text, width_constraint, height_constraint, &block)
        image = render_text(caption_text, width_constraint, &block)
        if height_constraint < image.rows
            percent = height_constraint.to_f / image.rows.to_f
            end_index = (caption_text.size * percent).to_i  # takes a leap into cropping
            image = render_text(caption_text[0..end_index] + "...", width_constraint, &block)
            while height_constraint < image.rows && end_index > 0 # reduce in big chunks until within range
                end_index -= 80
                image = render_text(caption_text[0..end_index] + "...", width_constraint, &block)
            end
            while height_constraint > image.rows                  # lengthen in smaller steps until exceed
                end_index += 10
                image = render_text(caption_text[0..end_index] + "...", width_constraint, &block)
            end
            while height_constraint < image.rows && end_index > 0 # reduce in baby steps until fit
                end_index -= 1
                image = render_text(caption_text[0..end_index] + "...", width_constraint, &block)
            end
        end
        image
    end

    def render_text(caption_text, width_constraint, &block)
        Magick::Image.read("caption:#{caption_text.to_s}") {
            # this wraps the text to fixed width
            self.size = width_constraint
            # other optional settings
            block.call(self) if block_given?
        }.first
    end
end

include RMagickTextUtil

# draw the backround image (simple black background)
RVG::dpi = 72
RVG.new(680.px, 480.px) do |canvas|
    canvas.background_fill = 'black'
end.draw.write('bg.png')

def processQuote(text, src)
	# draw the quote
	quote = render_cropped_text(text, 500, 430) do |img|
		img.fill = "#ffffff" # font color
		img.background_color = "transparent"
		img.pointsize = 23
		img.antialias = true
		img.font = "Helvetica"
	end
	quote.write "text.png"
	
	# draw the source
	source = render_cropped_text(src, 500, 40) do |img|
		img.fill = "#cccccc" # font color
		img.background_color = "transparent"
		img.pointsize = 18
		img.antialias = true
		img.font = "Helvetica"
	end
	source.write "source.png"	
end

doc = Hpricot.XML(open("http://igrigorik.tumblr.com/api/read?type=quote&num=5"))
(doc/'post').each do |post|
	text = CGI::unescapeHTML((post/'quote-text').inner_html).gsub(/&[^;]*;?/,'')
	source = CGI::unescapeHTML((post/'quote-source').inner_html).gsub(/\<.*?\>/, '').gsub(/&[^;]*;?/,'').strip
	
	puts "Processing: #{post['id']}\n"
	
	# render text
	processQuote(text, source)
		
	# Assemble the final image
	bg    = Image.read("bg.png").first
	quote = Image.read("quote.png").first
	text  = Image.read("text.png").first
	src   = Image.read("source.png").first
	
	# Set positions for each image
	quote.page = Rectangle.new(quote.rows,quote.columns,15,30)
	text.page  = Rectangle.new(text.rows,text.columns,105,65)
	src.page   = Rectangle.new(src.rows,src.columns,105,430) 
	
	# Render quote
	r = ImageList.new
	r << bg << quote << text << src
	r.flatten_images.write("quotes/#{post['id']}.jpg")

end